#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include <stdio.h>
#include <string.h>
#include <native_code_plugin.h>

#include <QtGui/QApplication>
#include <QtGui/QWidget>
#include <QtGui/QHBoxLayout>
#include <QtGui/QListWidget>
#include <QtGui/QListWidgetItem>
#include <QtGui/QMainWindow>
#include <QtGui/QKeyEvent>
#include <QtGui/qdesktopwidget.h>
#include <QtGui/qpixmapcache.h>
#include <QtGui/qscreen_qws.h>

#define STB_EXAMPLE_URLS "/config/stb_example_urls.txt"
#define WIDTH   1200
#define HEIGHT  700

#include <dune_stb_api.h>

extern "C"
{
#include <native_code_plugin.h>
}

static dune_stb_api_t *stb = NULL;

class MyMainWindow : public QMainWindow
{
public:
    MyMainWindow() :
        QMainWindow(0, Qt::SplashScreen),
        _playing(false), _paused(false)
    {
        setAttribute(Qt::WA_QuitOnClose, true);

        setAutoFillBackground(true);

        _list = new QListWidget();

        load_list();

        QRect rect = QApplication::desktop()->screenGeometry();
        _list->setFixedSize(WIDTH, HEIGHT);

        setCentralWidget(_list);
        setStyleSheet("* { background-color:rgb(0,0,0,0); padding: 10px ; font-size: 28pt ; }");
        _list->setStyleSheet("* { background-color:rgb(230,230,230); } ");
        setGeometry((rect.width() - WIDTH)/2, (rect.height() - HEIGHT)/2,
            WIDTH, HEIGHT);

        startTimer(500);
    }

    void load_list(void)
    {
        const char *file_name = STB_EXAMPLE_URLS;
        FILE *fp = fopen(file_name, "r");
        if (fp)
        {
            char buffer[PATH_MAX + 1];
            char *s, *url = NULL;
            QListWidgetItem *first = NULL;
            QListWidgetItem *item = NULL;
            do
            {
                buffer[PATH_MAX] = 0;
                url = fgets(buffer, sizeof(buffer), fp);
                if (!url)
                    break;
                if (url[PATH_MAX] != 0)
                {
                    fprintf(stderr, "Too long URL\n");
                    continue;
                }
                if (url[0] == 0)
                    continue;
                s = strchr(url, '\n');
                if (s)
                    *s = 0;
                item = new QListWidgetItem((const char *)url);
                _list->addItem(item);
                if (!first)
                    first = item;
            }while(1);
            fclose(fp);
            _list->setCurrentItem(first);
        }
        else
        {
            fprintf(stderr, "File: %s not found\n", file_name);
        }
    }

    void stop_playback()
    {
        dunestbapi_stop();
        _playing = false;
        _list->show();
    }

    void start_playback(const char *url)
    {
        fprintf(stderr, "Start playing URL: '%s'\n", url);
        dunestbapi_play(url);
        _list->hide();
        _playing = true;
    }

    void timerEvent(QTimerEvent *event)
    {
        static int counter = 0;
        if (!(++counter % 4))
            stb->log_rotate_if_needed();
        if (_playing)
        {
            dunestbapi_playbackState st;
            dunestbapi_getPlaybackState(&st);
            if (st < DUNESTBAPI_PLAYBACK_STATE_INITIALIZING ||
                st > DUNESTBAPI_PLAYBACK_STATE_BUFFERING)
                stop_playback();
        }
    }

    void keyPressEvent(QKeyEvent *event)
    {
        switch (event->key()) {
            case Qt::Key_Standby:
            case Qt::Key_MediaStop:
                fprintf(stderr, "Stop/Standby\n");
                if (!_playing)
                {
                    fprintf(stderr, "Close main window\n");
                    close();
                }
                else
                {
                    fprintf(stderr, "Stop/Standby in playback\n");
                    dunestbapi_stop();
                    _playing = false;
                    _list->show();
                }
                break;
            case Qt::Key_Down:
            case Qt::Key_Up:
                {
                    if (_playing)
                    {
                        fprintf(stderr, "Down/Up pressed in playback\n");
                        QListWidgetItem *item = _list->currentItem();
                        if (!item)
                            break;
                        int row = _list->currentRow();
                        int size = _list->count();
                        if (size == 0)
                            break;

                        if (event->key() == Qt::Key_Up)
                            --row;
                        else
                            ++row;
                        if (row < 0)
                            row = size - 1;
                        if (row >= size)
                            row = 0;

                        _list->setCurrentRow(row);
                        QListWidgetItem *new_item = _list->currentItem();
                        if (new_item && new_item != item)
                        {
                            item = new_item;
                            start_playback(item->text().toStdString().c_str());
                        }
                    }
                    else
                        QMainWindow::keyPressEvent(event);
                }
                break;
            case Qt::Key_Return:
            case Qt::Key_MediaPlay:
                {
                    fprintf(stderr, "Enter or Play\n");
                    if (_playing)
                    {
                        dunestbapi_playbackState st;
                        dunestbapi_getPlaybackState(&st);
                        if (st == DUNESTBAPI_PLAYBACK_STATE_PAUSED)
                        {
                            fprintf(stderr, "Resume\n");
                            dunestbapi_resume();
                        }
                        break;
                    }

                    QListWidgetItem *item = _list->currentItem();
                    if (!item)
                        break;

                    start_playback(item->text().toStdString().c_str());
                }
                break;
            case Qt::Key_Pause:
                {
                    fprintf(stderr, "Pause\n");
                    dunestbapi_playbackState st;
                    dunestbapi_getPlaybackState(&st);
                    switch (st)
                    {
                        case DUNESTBAPI_PLAYBACK_STATE_PLAYING:
                        case DUNESTBAPI_PLAYBACK_STATE_SEEKING:
                        case DUNESTBAPI_PLAYBACK_STATE_BUFFERING:
                            fprintf(stderr, "Pause\n");
                            dunestbapi_pause();
                            break;
                        case DUNESTBAPI_PLAYBACK_STATE_PAUSED:
                            fprintf(stderr, "Resume\n");
                            dunestbapi_resume();
                            break;
                        default:
                            break;
                    }
                }
                break;
            case Qt::Key_LaunchA:
            case Qt::Key_Escape:
            case Qt::Key_Backspace:
                {
                    fprintf(stderr, "Close and exit from plugin (all done)\n");
                    if (_playing)
                    {
                        dunestbapi_stop();
                        _playing = false;
                    }
                    close();
                }
                break;
            default:
                QMainWindow::keyPressEvent(event);
                break;
        }
    }

protected:
    QListWidget *_list;
    bool _playing;
    bool _paused;
};

static void dune_info(void)
{
    fprintf(stderr, "\n\n================================================\n\n");
    char *str;
    dunestbapi_getApiVersion(&str);
    fprintf(stderr, "    dunestbapi_getApiVersion:   %s\n", str);
    free(str);

    dunestbapi_getProductId(&str);
    fprintf(stderr, "    dunestbapi_getProductId:    %s\n", str);
    free(str);

    dunestbapi_getSerialNumber(&str);
    fprintf(stderr, "    dunestbapi_getSerialNumber: %s\n", str);
    free(str);

    dunestbapi_getFirmwareVersion(&str);
    fprintf(stderr, "    dunestbapi_getSerialNumber: %s\n", str);
    free(str);
    fprintf(stderr, "\n\n================================================\n\n");
}

/*************************************************************************/

static char *run_plugin(const char *arg)
{
    if (stb->enable_directfb_mode())
    {
        fprintf(stderr, "[dfb_test_plugin] stb dfb mode not supported\n");
        return strdup("Plugin requires DFB mode in STB");
    }

    dune_info();

    int argc = 2;
    char *argv[2];
    argv[0] = strdup("./stb");
    argv[1] = strdup("-qws");

    QApplication a(argc, argv);
    MyMainWindow w;
    w.show();
    //check power off
    a.exec();
    return 0;
}

/*************************************************************************/

extern "C"
{
    char *NATIVE_CODE_PLUGIN_RUN_FUNC(dune_stb_core_t *in, const char *arg);
}

/*************************************************************************/

char *NATIVE_CODE_PLUGIN_RUN_FUNC(dune_stb_core_t *in, const char *arg)
{
    stb = native_code_plugin_cast_stb(in);
    if (!stb)
        return strdup("unsupported stb");

    return run_plugin(arg);
}

/*************************************************************************/

