#ifndef DUNE_STB_API_H
#define DUNE_STB_API_H

#ifdef __cplusplus
extern "C"
{
#endif

///////////////////////////////////////////////////////////////////////////

// NOTES:
// - values of out parameters are undefined if function does not return
//   DUNESTBAPI_OK;
// - functions that return "type **" in out parameters allocate memory
//   dynamically. Caller is responsible for freeing the memory recursively
//   when it is not needed anymore with "free()". Utility functions from
//   section "Utility functions for various types" can used for this
//   purpose.

///////////////////////////////////////////////////////////////////////////

#define DUNESTBAPI_VERSION        0x0002
#define DUNESTBAPI_VERSION_STRING "0.2"

///////////////////////////////////////////////////////////////////////////

#define DUNESTBAPI_MAX_LANGUAGE_LENGTH   4
#define DUNESTBAPI_MAX_CODEC_LENGTH      32

typedef enum
{
    DUNESTBAPI_OK                  = 0,

    DUNESTBAPI_ALREADY_INITIALIZED = 1,
    DUNESTBAPI_NOT_INITIALIZED     = 2,
    DUNESTBAPI_POWER_OFF           = 3,

    DUNESTBAPI_ERROR               = 1000,
    DUNESTBAPI_INVALID_ARGUMENT    = 1001,
    DUNESTBAPI_NOT_SUPPORTED       = 1002
}
dunestbapi_result;

typedef enum
{
    DUNESTBAPI_PLAYBACK_STATE_STOPPED        = 1,
    DUNESTBAPI_PLAYBACK_STATE_INITIALIZING   = 2,
    DUNESTBAPI_PLAYBACK_STATE_PLAYING        = 3,
    DUNESTBAPI_PLAYBACK_STATE_PAUSED         = 4,
    DUNESTBAPI_PLAYBACK_STATE_SEEKING        = 5,
    DUNESTBAPI_PLAYBACK_STATE_BUFFERING      = 6,
    DUNESTBAPI_PLAYBACK_STATE_FINISHED       = 7,
    DUNESTBAPI_PLAYBACK_STATE_DEINITIALIZING = 8,

    DUNESTBAPI_PLAYBACK_STATE_NUMBER
}
dunestbapi_playbackState;

typedef enum
{
    DUNESTBAPI_PLAYBACK_EVENT_NO_EVENT                           = 1,
    DUNESTBAPI_PLAYBACK_EVENT_MEDIA_DESCRIPTION_CHANGED          = 2,
    DUNESTBAPI_PLAYBACK_EVENT_MEDIA_READ_STALLED                 = 3,
    DUNESTBAPI_PLAYBACK_EVENT_END_OF_MEDIA                       = 4,
    DUNESTBAPI_PLAYBACK_EVENT_ERROR_MEDIA_FORMAT_NOT_SUPPORTED   = 5,
    DUNESTBAPI_PLAYBACK_EVENT_ERROR_INTERNAL_ERROR               = 6,
    DUNESTBAPI_PLAYBACK_EVENT_ERROR_MEDIA_READ_FAILED            = 7,
    DUNESTBAPI_PLAYBACK_EVENT_ERROR_MEDIA_OPEN_FAILED            = 8,
    DUNESTBAPI_PLAYBACK_EVENT_EXTERNAL_ACTION                    = 9,
    DUNESTBAPI_PLAYBACK_EVENT_ERROR_MEDIA_PROTOCOL_NOT_SUPPORTED = 10,
    DUNESTBAPI_PLAYBACK_EVENT_ERROR_MEDIA_PERMISSION_DENIED      = 11,
    DUNESTBAPI_PLAYBACK_EVENT_SCRAMBLING_STATUS_CHANGED          = 12,

    DUNESTBAPI_PLAYBACK_EVENT_NUMBER
}
dunestbapi_playbackEvent;

#define DUNESTBAPI_SERVICE_TYPE_PLAYBACK  1
#define DUNESTBAPI_SERVICE_TYPE_PIP       2
#define DUNESTBAPI_SERVICE_TYPE_RECORDING 3

typedef struct
{
    int enableColorKey;
}
dunestbapi_initParameters;

typedef struct
{
    int index;
    int pid;
    char language[DUNESTBAPI_MAX_LANGUAGE_LENGTH];
    char codec[DUNESTBAPI_MAX_CODEC_LENGTH];
}
dunestbapi_audioTrack;

#define DUNESTBAPI_SUBTITLE_TYPE_NORMAL 0
#define DUNESTBAPI_SUBTITLE_TYPE_CC     1

typedef struct
{
    int index;
    int pid;
    char language[DUNESTBAPI_MAX_LANGUAGE_LENGTH];
    char codec[DUNESTBAPI_MAX_CODEC_LENGTH];
    int type;
}
dunestbapi_subtitleTrack;

typedef struct
{
    int connection;
    int mode;
    char *ipAddress;
    char *mask;
    char *gateway;
    char *dns1;
    char *dns2;
    char *wifiSsid;
    int wifiSecurity;
    char *wifiPassword;
    int wifiApMode;
    char *wifiApSsid;
    char *wifiApPassword;
    char *pppoeLogin;
    char *pppoePassword;
    int pppoeDnsMode;
    int pppoeDhcp;
}
dunestbapi_networkConfiguration;

typedef struct
{
    char *interface;
    char *ipAddress;
    char *netmask;
    char *macAddress;
    char *connection;
    int up;
    int running;
}
dunestbapi_interfaceStatus;

typedef struct
{
    char *ssid;
    int security;
}
dunestbapi_wifiNetwork;

typedef struct
{
    char *mediaUrl;
    char *path;
    char *uuid;
    char *label;
    char *name;
    long long size;
    long long free;
    char *fileSystem;
    int writable;
    int mounted;
}
dunestbapi_storage;

typedef void (*dunestbapi_storageCallback)(
    const char **in_mediaUrls,
    int in_mediaUrlsNumber);

typedef struct
{
    char *name;
    char *mediaUrl;
    char *path;
    int type;
    long long size;
    int modified;
    int accessed;
}
dunestbapi_folderItem;

typedef struct
{
    int status;
    int itemsNumber;
    dunestbapi_folderItem *items;
}
dunestbapi_folderContent;

typedef struct
{
    int startTime;
    int endTime;
}
dunestbapi_timeRange;

typedef struct
{
    int startTime;
    int endTime;
    char *name;
    char *description;
    int parentalRating;
}
dunestbapi_epgItem;

typedef struct
{
    char *provider;
    char *name;
    char *mediaUrl;
    int onid;
    int tsid;
    int sid;
    int lcn;
}
dunestbapi_channel;

typedef struct
{
    char *url;
    char *name;
}
dunestbapi_channelGroup;

typedef void (*dunestbapi_playbackEventCallback)(
    dunestbapi_playbackState in_previousState,
    dunestbapi_playbackState in_currentState,
    dunestbapi_playbackEvent in_lastEvent);

///////////////////////////////////////////////////////////////////////////

typedef enum
{
    DUNESTBAPI_RECORDING_STATE_SCHEDULED       = 1,
    DUNESTBAPI_RECORDING_STATE_STARTING        = 2,
    DUNESTBAPI_RECORDING_STATE_RECORDING       = 3,
    DUNESTBAPI_RECORDING_STATE_COMPLETE        = 4,
    DUNESTBAPI_RECORDING_STATE_INCOMPLETE      = 5,
    DUNESTBAPI_RECORDING_STATE_STOPPED         = 6,
    DUNESTBAPI_RECORDING_STATE_ERROR           = 7,

    DUNESTBAPI_RECORDING_STATE_NUMBER
}
dunestbapi_recordingState;

typedef enum
{
    DUNESTBAPI_RECORDING_ERROR_NONE                  = 1,
    DUNESTBAPI_RECORDING_ERROR_GENERIC               = 2,
    DUNESTBAPI_RECORDING_ERROR_STORAGE_NOT_EXIST     = 3,
    DUNESTBAPI_RECORDING_ERROR_INSUFFICIENT_STORAGE  = 4,
    DUNESTBAPI_RECORDING_ERROR_RESOURCE_LIMITATION   = 5,
    DUNESTBAPI_RECORDING_ERROR_NO_PERMISSION         = 6,
    DUNESTBAPI_RECORDING_ERROR_CONFLICT              = 7,
    DUNESTBAPI_RECORDING_ERROR_NOT_SUPPORTED         = 8,
    DUNESTBAPI_RECORDING_ERROR_STORAGE_ERROR         = 9,

    DUNESTBAPI_RECORDING_ERROR_NUMBER
}
dunestbapi_recordingError;

#define DUNESTBAPI_RECORDING_STATE_MASK_SCHEDULED   \
    (1 << DUNESTBAPI_RECORDING_STATE_SCHEDULED)
#define DUNESTBAPI_RECORDING_STATE_MASK_STARTING    \
    (1 << DUNESTBAPI_RECORDING_STATE_STARTING)
#define DUNESTBAPI_RECORDING_STATE_MASK_RECORDING   \
    (1 << DUNESTBAPI_RECORDING_STATE_RECORDING)
#define DUNESTBAPI_RECORDING_STATE_MASK_COMPLETE    \
    (1 << DUNESTBAPI_RECORDING_STATE_COMPLETE)
#define DUNESTBAPI_RECORDING_STATE_MASK_INCOMPLETE  \
    (1 << DUNESTBAPI_RECORDING_STATE_INCOMPLETE)
#define DUNESTBAPI_RECORDING_STATE_MASK_STOPPED     \
    (1 << DUNESTBAPI_RECORDING_STATE_STOPPED)
#define DUNESTBAPI_RECORDING_STATE_MASK_ERROR       \
    (1 << DUNESTBAPI_RECORDING_STATE_ERROR)

#define DUNESTBAPI_RECORDING_STATE_MASK_ANY         0

typedef struct
{
    int stateMask;
}
dunestbapi_recordingFilter;

typedef struct
{
    char *id;
    char *sourceMediaUrl;
    int startTime;              // 0 means right now
    int duration;               // -1 means infinite
    char *title;
    char *description;
    char *metadata;
}
dunestbapi_recordingParameters;

typedef struct
{
    dunestbapi_recordingParameters params;
    dunestbapi_recordingState state;
    dunestbapi_recordingError error;
    char *mediaUrl;
    int actualStartTime;       // in seconds since EPOCH
    int actualDuration;        // in seconds
    long long storageSize;     // in bytes
}
dunestbapi_recordingStatus;

typedef void (*dunestbapi_recordingStateCallback)(
    const char *storageMediaUrl,
    const char *recordingId,
    dunestbapi_recordingState previousState,
    dunestbapi_recordingState currentState);

///////////////////////////////////////////////////////////////////////////

typedef enum
{
    DUNESTBAPI_DVB_SCANNER_STATE_STOPPED        = 1,
    DUNESTBAPI_DVB_SCANNER_STATE_INITIALIZING   = 2,
    DUNESTBAPI_DVB_SCANNER_STATE_SCANNING       = 3,
    DUNESTBAPI_DVB_SCANNER_STATE_SUCCEEDED      = 4,
    DUNESTBAPI_DVB_SCANNER_STATE_FAILED         = 5,
    DUNESTBAPI_DVB_SCANNER_STATE_CANCELLED      = 6,

    DUNESTBAPI_DVB_SCANNER_STATE_NUMBER
}
dunestbapi_dvbScannerState;

typedef struct
{
    dunestbapi_dvbScannerState state;

    int totalSatellitesNumber;
    int totalTranspondersNumber;

    int scannedSatellitesNumber;
    int scannedTranspondersNumber;

    int foundChannelsNumber;

    int currentTransponderFrequency;
    int currentTransponderPolarization;
}
dunestbapi_dvbsScannerStatus;

typedef void (*dunestbapi_dvbsScannerCallback)(
    const dunestbapi_dvbsScannerStatus *in_dvbsScannerStatus);

typedef struct
{
    dunestbapi_dvbScannerState state;

    int totalTranspondersNumber;
    int scannedTranspondersNumber;

    int foundChannelsNumber;

    int currentTransponderFrequency;
}
dunestbapi_dvbcScannerStatus;

typedef void (*dunestbapi_dvbcScannerCallback)(
    const dunestbapi_dvbcScannerStatus *in_dvbcScannerStatus);

typedef struct
{
    int areaX;
    int areaY;
    int areaWidth;
    int areaHeight;
    int imageWidth;
    int imageHeight;
    char *imageName;
}
dunestbapi_saveVideoPictureParameters;

///////////////////////////////////////////////////////////////////////////

// Should not be called in context of native code plugin usage.
dunestbapi_result dunestbapi_init(int *inout_argc, char **inout_argv);
dunestbapi_result dunestbapi_initExt(int *inout_argc, char **inout_argv,
    const dunestbapi_initParameters in_parameters);

// Should not be called in context of native code plugin usage.
void dunestbapi_deinit(void);

// Launches native Dune UI.
// Should not be called in context of native code plugin usage.
dunestbapi_result dunestbapi_nativeUi(void);

// Launches native Dune UI with given URL.
// Should not be called in context of native code plugin usage.
dunestbapi_result dunestbapi_nativeUiExt(const char *in_url);

#define DUNESTBAPI_EXIT_MODE_NORMAL    0
#define DUNESTBAPI_EXIT_MODE_STANDBY   1
#define DUNESTBAPI_EXIT_MODE_POWER_OFF 2

dunestbapi_result dunestbapi_getExitMode(int *out_mode);

dunestbapi_result dunestbapi_setExitMode(const int in_mode);

#define DUNESTBAPI_STANDBY_MODE_OFF 0
#define DUNESTBAPI_STANDBY_MODE_ON  1

dunestbapi_result dunestbapi_getStandbyMode(int *out_mode);

dunestbapi_result dunestbapi_setStandbyMode(const int in_mode);

///////////////////////////////////////////////////////////////////////////

dunestbapi_result dunestbapi_getApplicationId(char **out_applicationId);

dunestbapi_result dunestbapi_setApplicationId(
    const char *in_applicationId);

///////////////////////////////////////////////////////////////////////////

dunestbapi_result dunestbapi_getApiVersion(char **out_apiVersion);

dunestbapi_result dunestbapi_getProductId(char **out_productId);

dunestbapi_result dunestbapi_getSerialNumber(char **out_serialNumber);

dunestbapi_result dunestbapi_getFirmwareVersion(
    char **out_firmwareVersion);

dunestbapi_result dunestbapi_getMacAddress(char **out_macAddress);

dunestbapi_result dunestbapi_getPrimaryMacAddress(char **out_macAddress);

dunestbapi_result dunestbapi_getIpAddress(char **out_ipAddress);

dunestbapi_result dunestbapi_getDns1Address(char **out_ipAddress);

dunestbapi_result dunestbapi_getDns2Address(char **out_ipAddress);

dunestbapi_result dunestbapi_getGatewayAddress(char **out_ipAddress);

dunestbapi_result dunestbapi_log(const char *in_message);

dunestbapi_result dunestbapi_reboot(void);

dunestbapi_result dunestbapi_fileExists(const char *in_path,
    int *out_exists);

dunestbapi_result dunestbapi_getUserSetting(const char *in_key,
    char **out_value);

dunestbapi_result dunestbapi_setUserSetting(const char *in_key,
    const char *in_value);

dunestbapi_result dunestbapi_getHomePage(char **out_homePage);

dunestbapi_result dunestbapi_getFirmwareUpgradeUrl(char **out_url);

dunestbapi_result dunestbapi_getFirmwareUpgradeVersion(char **out_version);

dunestbapi_result dunestbapi_startFirmwareUpgrade(const char *in_url,
    const char *in_version);

#define DUNESTBAPI_RESET_SETTINGS_FULL 0
#define DUNESTBAPI_RESET_SETTINGS_SOFT 1

dunestbapi_result dunestbapi_resetSettings(const int in_reset);

///////////////////////////////////////////////////////////////////////////

dunestbapi_result dunestbapi_getPlaybackState(
    dunestbapi_playbackState *out_playbackState);

dunestbapi_result dunestbapi_setInitialSpeed(const int in_speed);

dunestbapi_result dunestbapi_setInitialPositionInSeconds(
    const int in_position);

dunestbapi_result dunestbapi_setInitialAudioPid(const int in_audioPid);

dunestbapi_result dunestbapi_play(const char *in_mediaUrl);

dunestbapi_result dunestbapi_stop(void);

dunestbapi_result dunestbapi_canPause(int *out_canPause);

dunestbapi_result dunestbapi_pause(void);

dunestbapi_result dunestbapi_resume(void);

dunestbapi_result dunestbapi_getSpeed(int *out_speed);

dunestbapi_result dunestbapi_setSpeed(const int in_speed);

dunestbapi_result dunestbapi_hasLength(int *out_hasLength);

dunestbapi_result dunestbapi_getLengthInSeconds(int *out_length);

dunestbapi_result dunestbapi_getPositionInSeconds(int *out_position);

dunestbapi_result dunestbapi_setPositionInSeconds(const int in_position);

dunestbapi_result dunestbapi_setSpeedAndPositionInSeconds(
    const int in_speed, const int in_position);

dunestbapi_result dunestbapi_isScramblingDetected(int *out_scramblingDetected);

dunestbapi_result dunestbapi_isVideoEnabled(int *out_videoEnabled);

dunestbapi_result dunestbapi_enableVideo(void);

dunestbapi_result dunestbapi_disableVideo(void);

dunestbapi_result dunestbapi_isVideoOnTop(int *out_videoOnTop);

dunestbapi_result dunestbapi_enableVideoOnTop(void);

dunestbapi_result dunestbapi_disableVideoOnTop(void);

dunestbapi_result dunestbapi_getVideoZOrder(int *out_zOrder);

dunestbapi_result dunestbapi_setVideoZOrder(const int in_zOrder);

dunestbapi_result dunestbapi_getOSDZOrder(int *out_zOrder);

dunestbapi_result dunestbapi_setOSDZOrder(const int in_zOrder);

dunestbapi_result dunestbapi_isWindowFullScreen(int *out_windowFullScreen);

dunestbapi_result dunestbapi_enableWindowFullScreen(void);

dunestbapi_result dunestbapi_getWindowRect(int *out_x, int *out_y,
    int *out_width, int *out_height);

dunestbapi_result dunestbapi_setWindowRect(const int in_x, const int in_y,
    const int in_width, const int in_height);

dunestbapi_result dunestbapi_getClipRect(int *out_x, int *out_y,
    int *out_width, int *out_height);

dunestbapi_result dunestbapi_setClipRect(const int in_x, const int in_y,
    const int in_width, const int in_height);

dunestbapi_result dunestbapi_getVideoSourceRect(int *out_x, int *out_y,
    int *out_width, int *out_height);

dunestbapi_result dunestbapi_setVideoSourceRect(const int in_x, const int in_y,
    const int in_width, const int in_height);

/**
    Saves specified area of playing video to BMP image file with
    specified dimensions. Video area coordinates are relative to
    video. Relative coordinates have a range from 0 to 4096 where [0,0]
    specifies the left top corner of video and [4096,4096] specifies
    the bottom right corner. If area width or height are zero
    they will be recalculated from the playing video sizes.
    If image width or height are specified and not equal to area
    width or height scaling is performed.
    Resulting image is stored in file /tmp/screenshot/image{imageName}.bmp.
    The file is intended for processing by separate application working on
    STB. The application is also responsible for deletion of image files to
    preserve memory consumption.
**/
dunestbapi_result dunestbapi_saveVideoPicture(
    const dunestbapi_saveVideoPictureParameters *in_parameters);

#define DUNESTBAPI_VIDEO_ZOOM_NORMAL                            0
#define DUNESTBAPI_VIDEO_ZOOM_ENLARGE                           1
#define DUNESTBAPI_VIDEO_ZOOM_MAKE_WIDER                        2
#define DUNESTBAPI_VIDEO_ZOOM_NON_LINEAR_STRETCH                3
#define DUNESTBAPI_VIDEO_ZOOM_NON_LINEAR_STRETCH_TO_FULL_SCREEN 4
#define DUNESTBAPI_VIDEO_ZOOM_MAKE_TALLER                       5
#define DUNESTBAPI_VIDEO_ZOOM_CUT_EDGES                         6
#define DUNESTBAPI_VIDEO_ZOOM_FULL_SCREEN                       8
#define DUNESTBAPI_VIDEO_ZOOM_STRETCH_TO_FULL_SCREEN            9

dunestbapi_result dunestbapi_getVideoZoom(int *out_zoom);

dunestbapi_result dunestbapi_setVideoZoom(const int in_zoom);

#define DUNESTBAPI_VIDEO_CONNECTOR_HDMI      0
#define DUNESTBAPI_VIDEO_CONNECTOR_COMPONENT 1
#define DUNESTBAPI_VIDEO_CONNECTOR_COMPOSITE 2
#define DUNESTBAPI_VIDEO_CONNECTOR_SVIDEO    3

dunestbapi_result dunestbapi_getVideoConnector(int *out_connector);

dunestbapi_result dunestbapi_setVideoConnector(const int in_connector);

#define DUNESTBAPI_VIDEO_MODE_NTSC    0
#define DUNESTBAPI_VIDEO_MODE_480P    1
#define DUNESTBAPI_VIDEO_MODE_PAL     2
#define DUNESTBAPI_VIDEO_MODE_576P    3
#define DUNESTBAPI_VIDEO_MODE_720P50  4
#define DUNESTBAPI_VIDEO_MODE_720P60  5
#define DUNESTBAPI_VIDEO_MODE_1080I50 6
#define DUNESTBAPI_VIDEO_MODE_1080I60 7
#define DUNESTBAPI_VIDEO_MODE_1080P23 8
#define DUNESTBAPI_VIDEO_MODE_1080P24 9
#define DUNESTBAPI_VIDEO_MODE_1080P50 10
#define DUNESTBAPI_VIDEO_MODE_1080P60 11

dunestbapi_result dunestbapi_getVideoMode(int *out_mode);

dunestbapi_result dunestbapi_setVideoMode(const int in_mode);

#define DUNESTBAPI_COMPONENT_OUTPUT_MODE_HD    0
#define DUNESTBAPI_COMPONENT_OUTPUT_MODE_SD    1
#define DUNESTBAPI_COMPONENT_OUTPUT_MODE_NONE  2

dunestbapi_result dunestbapi_getComponentOutputMode(int *out_mode);

dunestbapi_result dunestbapi_setComponentOutputMode(const int in_mode);

#define DUNESTBAPI_ASPECT_RATIO_AUTO 0
#define DUNESTBAPI_ASPECT_RATIO_16_9 1
#define DUNESTBAPI_ASPECT_RATIO_4_3  2

dunestbapi_result dunestbapi_getAspectRatio(int *out_aspect);

dunestbapi_result dunestbapi_setAspectRatio(const int in_aspect);

#define DUNESTBAPI_SCREEN_SIZE_AUTO      0
#define DUNESTBAPI_SCREEN_SIZE_720_480   1
#define DUNESTBAPI_SCREEN_SIZE_720_576   2
#define DUNESTBAPI_SCREEN_SIZE_1280_720  3
#define DUNESTBAPI_SCREEN_SIZE_1920_1080 4

dunestbapi_result dunestbapi_getScreenSize(int *out_size);

dunestbapi_result dunestbapi_setScreenSize(const int in_size);

dunestbapi_result dunestbapi_getColorKey(int *out_r, int *out_g,
    int *out_b);

dunestbapi_result dunestbapi_setColorKey(const int in_r, const int in_g,
    const int in_b);

dunestbapi_result dunestbapi_getVideoSize(int *out_width, int *out_height);

dunestbapi_result dunestbapi_getRcAutoRepeatPeriod(int *out_period);

dunestbapi_result dunestbapi_setRcAutoRepeatPeriod(const int in_period);

#define DUNESTBAPI_KEY_UNKNOWN    0
#define DUNESTBAPI_KEY_RETURN     8
#define DUNESTBAPI_KEY_CLEAR     12
#define DUNESTBAPI_KEY_ENTER     13
#define DUNESTBAPI_KEY_POPUP     18
#define DUNESTBAPI_KEY_PAUSE     19
#define DUNESTBAPI_KEY_PPLUS     33
#define DUNESTBAPI_KEY_PMINUS    34
#define DUNESTBAPI_KEY_LEFT      37
#define DUNESTBAPI_KEY_UP        38
#define DUNESTBAPI_KEY_RIGHT     39
#define DUNESTBAPI_KEY_DOWN      40
#define DUNESTBAPI_KEY_SELECT    41
#define DUNESTBAPI_KEY_0         48
#define DUNESTBAPI_KEY_1         49
#define DUNESTBAPI_KEY_2         50
#define DUNESTBAPI_KEY_3         51
#define DUNESTBAPI_KEY_4         52
#define DUNESTBAPI_KEY_5         53
#define DUNESTBAPI_KEY_6         54
#define DUNESTBAPI_KEY_7         55
#define DUNESTBAPI_KEY_8         56
#define DUNESTBAPI_KEY_9         57
#define DUNESTBAPI_KEY_ANGLE    168
#define DUNESTBAPI_KEY_SEARCH   170
#define DUNESTBAPI_KEY_MUTE     173
#define DUNESTBAPI_KEY_VMINUS   174
#define DUNESTBAPI_KEY_VPLUS    175
#define DUNESTBAPI_KEY_NEXT     176
#define DUNESTBAPI_KEY_PREV     177
#define DUNESTBAPI_KEY_STOP     178
#define DUNESTBAPI_KEY_A        193
#define DUNESTBAPI_KEY_B        194
#define DUNESTBAPI_KEY_C        195
#define DUNESTBAPI_KEY_D        196
#define DUNESTBAPI_KEY_URL      197
#define DUNESTBAPI_KEY_EJECT    198
#define DUNESTBAPI_KEY_INFO     199
#define DUNESTBAPI_KEY_TOP_MENU 200
#define DUNESTBAPI_KEY_SETUP    201
#define DUNESTBAPI_KEY_MODE     202
#define DUNESTBAPI_KEY_SLOW     203
#define DUNESTBAPI_KEY_REW      204
#define DUNESTBAPI_KEY_FWD      205
#define DUNESTBAPI_KEY_SUBTITLE 206
#define DUNESTBAPI_KEY_AUDIO    207
#define DUNESTBAPI_KEY_REC      208
#define DUNESTBAPI_KEY_DUNE     209
#define DUNESTBAPI_KEY_PLAY     250
#define DUNESTBAPI_KEY_ZOOM     251

dunestbapi_result dunestbapi_getRcAutoRepeatKeys(int **out_keys,
    int *out_keysNumber);

dunestbapi_result dunestbapi_setRcAutoRepeatKeys(const int *in_keys,
    const int in_keysNumber);

///////////////////////////////////////////////////////////////////////////

dunestbapi_result dunestbapi_getVolume(int *out_volume);

dunestbapi_result dunestbapi_setVolume(const int in_volume);

dunestbapi_result dunestbapi_isMuteEnabled(int *out_mute);

dunestbapi_result dunestbapi_enableMute(void);

dunestbapi_result dunestbapi_disableMute(void);

dunestbapi_result dunestbapi_getAudioTrack(int *out_audioTrack);

dunestbapi_result dunestbapi_setAudioTrack(const int in_audioTrack);

dunestbapi_result dunestbapi_getAudioTracksDescription(
    dunestbapi_audioTrack **out_audioTracksDescription,
    int *out_audioTracksNumber);

#define DUNESTBAPI_DIGITAL_AUDIO_OUTPUT_PCM       0
#define DUNESTBAPI_DIGITAL_AUDIO_OUTPUT_BITSTREAM 1

dunestbapi_result dunestbapi_getDigitalAudioOutput(int *out_output);

dunestbapi_result dunestbapi_setDigitalAudioOutput(const int in_output);

dunestbapi_result dunestbapi_getDefaultAudioLanguage(char **out_language);

dunestbapi_result dunestbapi_setDefaultAudioLanguage(
    const char *in_language);

///////////////////////////////////////////////////////////////////////////

dunestbapi_result dunestbapi_getSubtitleTrack(int *out_subtitleTrack);

dunestbapi_result dunestbapi_setSubtitleTrack(const int in_subtitleTrack);

dunestbapi_result dunestbapi_getSubtitleTracksDescription(
    dunestbapi_subtitleTrack **out_subtitleTracksDescription,
    int *out_subtitleTracksNumber);

dunestbapi_result dunestbapi_disableSubtitleTrack(void);

dunestbapi_result dunestbapi_getDefaultSubtitleLanguage(
    char **out_language);

dunestbapi_result dunestbapi_setDefaultSubtitleLanguage(
    const char *in_language);

///////////////////////////////////////////////////////////////////////////

dunestbapi_result dunestbapi_isTeletextAvailable(
    int *out_teletextAvailable);

dunestbapi_result dunestbapi_isTeletextEnabled(int *out_teletextEnabled);

dunestbapi_result dunestbapi_enableTeletext(void);

dunestbapi_result dunestbapi_disableTeletext(void);

dunestbapi_result dunestbapi_getTeletextPageNumber(
    int *out_teletextPageNumber);

dunestbapi_result dunestbapi_setTeletextPageNumber(
    const int in_teletextPageNumber);

dunestbapi_result dunestbapi_isTeletextMixModeEnabled(
    int *out_teletextMixModeEnabled);

dunestbapi_result dunestbapi_enableTeletextMixMode(void);

dunestbapi_result dunestbapi_disableTeletextMixMode(void);

dunestbapi_result dunestbapi_inputTeletextKey(const int in_key);

///////////////////////////////////////////////////////////////////////////

#define DUNESTBAPI_NETWORK_CONNECTION_ETHERNET      0
#define DUNESTBAPI_NETWORK_CONNECTION_GIGAETH       1
#define DUNESTBAPI_NETWORK_CONNECTION_WIFI          2
#define DUNESTBAPI_NETWORK_CONNECTION_PPPOE         3
#define DUNESTBAPI_NETWORK_CONNECTION_NONE          4

#define DUNESTBAPI_NETWORK_MODE_AUTO                0
#define DUNESTBAPI_NETWORK_MODE_AUTO_MANUAL_DNS     1
#define DUNESTBAPI_NETWORK_MODE_MANUAL              2

#define DUNESTBAPI_WIFI_SECURITY_NONE               0
#define DUNESTBAPI_WIFI_SECURITY_WEP_64_BIT_ASCII   1
#define DUNESTBAPI_WIFI_SECURITY_WEP_64_BIT_HEX     2
#define DUNESTBAPI_WIFI_SECURITY_WEP_128_BIT_ASCII  3
#define DUNESTBAPI_WIFI_SECURITY_WEP_128_BIT_HEX    4
#define DUNESTBAPI_WIFI_SECURITY_WPA_TKIP           5
#define DUNESTBAPI_WIFI_SECURITY_WPA_AES            6
#define DUNESTBAPI_WIFI_SECURITY_WPA2_TKIP          7
#define DUNESTBAPI_WIFI_SECURITY_WPA2_AES           8

#define DUNESTBAPI_WIFI_AP_MODE_DISABLED            0
#define DUNESTBAPI_WIFI_AP_MODE_ROUTER              1

#define DUNESTBAPI_PPPOE_DNS_MODE_AUTO              0
#define DUNESTBAPI_PPPOE_DNS_MODE_MANUAL            1

#define DUNESTBAPI_PPPOE_DHCP_DISABLED              0
#define DUNESTBAPI_PPPOE_DHCP_ENABLED               1

dunestbapi_result dunestbapi_setNetworkConfiguration(
    const dunestbapi_networkConfiguration *in_networkConfiguration);

dunestbapi_result dunestbapi_getNetworkConfiguration(
    dunestbapi_networkConfiguration **out_networkConfiguration);

dunestbapi_result dunestbapi_getNetworkStatus(
    dunestbapi_interfaceStatus **out_interfaceStatus,
    int *out_interfacesNumber);

dunestbapi_result dunestbapi_scanForWifiNetworks(const int in_timeout);

dunestbapi_result dunestbapi_getWifiNetworksDescription(
    dunestbapi_wifiNetwork **out_wifiNetworksDescription,
    int *out_wifiNetworksNumber);

///////////////////////////////////////////////////////////////////////////

#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_12_00 0
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_11_00 1
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_10_00 2
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_09_00 3
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_08_00 4
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_07_00 5
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_06_00 6
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_05_00 7
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_04_00 8
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_03_30 9
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_03_00 10
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_02_00 11
#define DUNESTBAPI_TIME_ZONE_GMT_MINUS_01_00 12
#define DUNESTBAPI_TIME_ZONE_GMT             13
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_01_00  14
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_02_00  15
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_03_00  16
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_03_30  17
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_04_00  18
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_04_30  19
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_05_00  20
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_05_30  21
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_05_45  22
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_06_00  23
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_06_30  24
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_07_00  25
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_08_00  26
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_09_00  27
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_09_30  28
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_10_00  29
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_11_00  30
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_12_00  31
#define DUNESTBAPI_TIME_ZONE_GMT_PLUS_13_00  32
#define DUNESTBAPI_TIME_ZONE_AUTO            33

dunestbapi_result dunestbapi_getTimeZone(int *out_zone);

dunestbapi_result dunestbapi_setTimeZone(const int in_zone);

#define DUNESTBAPI_DAYLIGHT_SAVING_TIME_OFF  0
#define DUNESTBAPI_DAYLIGHT_SAVING_TIME_AUTO 1

dunestbapi_result dunestbapi_getDaylightSavingTime(int *out_time);

dunestbapi_result dunestbapi_setDaylightSavingTime(const int in_time);

///////////////////////////////////////////////////////////////////////////

dunestbapi_result dunestbapi_isStorageAvailable(int *out_available);

dunestbapi_result dunestbapi_getMainStorageDescription(
    dunestbapi_storage **out_storageDescription);

dunestbapi_result dunestbapi_getStorages(
    char ***out_mediaUrls,
    int *out_mediaUrlsNumber);

dunestbapi_result dunestbapi_getStorageDescription(
    const char *in_mediaUrl,
    dunestbapi_storage **out_storageDescription);

dunestbapi_result dunestbapi_setStorageCallback(
    dunestbapi_storageCallback in_callback);

dunestbapi_result dunestbapi_getFolderItemsNumber(const char *in_mediaUrl,
    int *out_number);

#define DUNESTBAPI_CONTENT_TYPE_DIRECTORY 0
#define DUNESTBAPI_CONTENT_TYPE_FILE      1
#define DUNESTBAPI_CONTENT_TYPE_VIDEO     2
#define DUNESTBAPI_CONTENT_TYPE_AUDIO     3
#define DUNESTBAPI_CONTENT_TYPE_IMAGE     4

dunestbapi_result dunestbapi_getFolderContent(const char *in_mediaUrl,
    const int in_startIndex, const int in_maxCount,
    dunestbapi_folderContent **out_folderContentDescription);

///////////////////////////////////////////////////////////////////////////

// The callback is called on internal STB thread.
// Pass NULL to disable playback event callback.
dunestbapi_result dunestbapi_setPlaybackEventCallback(
    dunestbapi_playbackEventCallback in_callback);

///////////////////////////////////////////////////////////////////////////

//
// PVR.
//

dunestbapi_result dunestbapi_getMaxConcurrentRecordings(int *out_number);

dunestbapi_result dunestbapi_createRecording(
    const char *in_storageMediaUrl,
    const dunestbapi_recordingParameters *in_recordingParameters);

dunestbapi_result dunestbapi_stopRecording(
    const char *in_storageMediaUrl,
    const char *in_recordingId);

dunestbapi_result dunestbapi_deleteRecording(
    const char *in_storageMediaUrl,
    const char *in_recordingId);

dunestbapi_result dunestbapi_getRecordingStatus(
    const char *in_storageMediaUrl,
    const char *in_recordingId,
    dunestbapi_recordingStatus **out_recordingStatus);

dunestbapi_result dunestbapi_getRecordingsNumber(
    const char *in_storageMediaUrl,
    const dunestbapi_recordingFilter *in_filter,
    int *out_recordingsNumber);

dunestbapi_result dunestbapi_getRecordingsDescription(
    const char *in_storageMediaUrl,
    const dunestbapi_recordingFilter *in_filter,
    const int in_startIndex,
    const int in_maxCount,
    dunestbapi_recordingStatus **out_recordings,
    int *out_recordingsNumber);

dunestbapi_result dunestbapi_setRecordingStateCallback(
    dunestbapi_recordingStateCallback in_callback);

///////////////////////////////////////////////////////////////////////////

//
// Verimatrix IPTV.
//

dunestbapi_result dunestbapi_setVMIptvCompanyName(const char *in_name);
dunestbapi_result dunestbapi_setVMIptvServerAddress(const char *in_address);
dunestbapi_result dunestbapi_setVMIptvServerPort(const int in_port);
dunestbapi_result dunestbapi_setVMIptvRootCert(const char *in_cert_path);
dunestbapi_result dunestbapi_setVMIptvPreferredVKS(const char *in_address_and_port);
dunestbapi_result dunestbapi_setVMIptvTimeout(const int in_timeout);
dunestbapi_result dunestbapi_setVMIptvErrorLevel(int in_level);
dunestbapi_result dunestbapi_setVMIptvDisableLog(int in_disabled);
dunestbapi_result dunestbapi_setVMIptvMinKeyRetryInterval(int in_interval);
dunestbapi_result dunestbapi_setVMIptvMessageFormat(const char *in_format);
dunestbapi_result dunestbapi_setVMIptvConnectionRetries(int in_num_retries);

dunestbapi_result dunestbapi_configureVMIptv(void);

typedef enum
{
    DUNESTBAPI_VMIPTV_STATE_NOT_INITIALIZED = 1,
    DUNESTBAPI_VMIPTV_STATE_INITIALIZING    = 2,
    DUNESTBAPI_VMIPTV_STATE_INITIALIZED     = 3,
    DUNESTBAPI_VMIPTV_STATE_DEINITIALIZING  = 4,

    DUNESTBAPI_VMIPTV_STATE_NUMBER
}
dunestbapi_vmIptvState;

typedef enum
{
    DUNESTBAPI_VMIPTV_EVENT_NO_EVENT                = 1,

    // Verimatrix IPTV client errors: ERROR_HTTPCON_NOROOTCERT
    DUNESTBAPI_VMIPTV_EVENT_ERROR_INVALID_CONFIG    = 2,

    // Verimatrix IPTV client errors: ERROR_HTTPCON_INITCLIENT_CLEAR,
    // ERROR_HTTPCON_SSLCONNECT.
    DUNESTBAPI_VMIPTV_EVENT_ERROR_CONNECTION_FAILED = 3,

    // Verimatrix IPTV client decode result: eVmDecodeFailurePinRequired,
    // eVmDecodeFailureKeyNotAvailable.
    DUNESTBAPI_VMIPTV_EVENT_ERROR_NO_PERMISSION     = 4,

    // Other Verimatrix IPTV client errors and decode results.
    DUNESTBAPI_VMIPTV_EVENT_ERROR_OTHER             = 5,

    DUNESTBAPI_VMIPTV_EVENT_NUMBER
}
dunestbapi_vmIptvEvent;

// The in_vmiptv_error_code parameter contains Verimatrix IPTV client error
// code or decode result, or 0 if there is no Verimatrix IPTV client error.
typedef void (*dunestbapi_vmIptvEventCallback)(
    dunestbapi_vmIptvState in_previousState,
    dunestbapi_vmIptvState in_currentState,
    dunestbapi_vmIptvEvent in_event,
    int in_vmiptv_error_code);

// The callback may be called in other thread.
// Pass NULL to disable callback.
dunestbapi_result dunestbapi_setVMIptvEventCallback(
    dunestbapi_vmIptvEventCallback in_callback);

dunestbapi_result dunestbapi_startVMIptvInitialization(void);

///////////////////////////////////////////////////////////////////////////

//
// Verimatrix Web.
//

dunestbapi_result dunestbapi_setVMWebBootAddress(const char *in_address);
dunestbapi_result dunestbapi_setVMWebCompanyName(const char *in_name);

dunestbapi_result dunestbapi_configureVMWeb(void);

typedef enum
{
    DUNESTBAPI_VMWEB_STATE_NOT_INITIALIZED = 1,
    DUNESTBAPI_VMWEB_STATE_INITIALIZING    = 2,
    DUNESTBAPI_VMWEB_STATE_INITIALIZED     = 3,
    DUNESTBAPI_VMWEB_STATE_DEINITIALIZING  = 4,

    DUNESTBAPI_VMWEB_STATE_NUMBER
}
dunestbapi_vmWebState;

typedef enum
{
    DUNESTBAPI_VMWEB_EVENT_NO_EVENT                = 1,

    // Verimatrix Web client errors: BadINI
    DUNESTBAPI_VMWEB_EVENT_ERROR_INVALID_CONFIG    = 2,

    // Verimatrix Web client errors: NoConnect, CantRetrieveBoot
    DUNESTBAPI_VMWEB_EVENT_ERROR_CONNECTION_FAILED = 3,

    // Verimatrix Web client errors: NotEntitled, CantProvision,
    // NotProvisioned, KeyFileNotEntitled, CertificateExpired
    DUNESTBAPI_VMWEB_EVENT_ERROR_NO_PERMISSION     = 4,

    // Other Verimatrix Web client errors.
    DUNESTBAPI_VMWEB_EVENT_ERROR_OTHER             = 5,

    DUNESTBAPI_VMWEB_EVENT_NUMBER
}
dunestbapi_vmWebEvent;

// The in_vmweb_error_code parameter contains Verimatrix Web client
// error, or 0 if there is no Verimatrix Web client error.
typedef void (*dunestbapi_vmWebEventCallback)(
    dunestbapi_vmWebState in_previousState,
    dunestbapi_vmWebState in_currentState,
    dunestbapi_vmWebEvent in_event,
    int in_vmweb_error_code);

// The callback may be called in other thread.
// Pass NULL to disable callback.
dunestbapi_result dunestbapi_setVMWebEventCallback(
    dunestbapi_vmWebEventCallback in_callback);

dunestbapi_result dunestbapi_startVMWebInitialization(void);

///////////////////////////////////////////////////////////////////////////

//
// Verimatrix DVB.
//

typedef enum
{
    DUNESTBAPI_VMDVB_STATE_NOT_INITIALIZED = 1,
    DUNESTBAPI_VMDVB_STATE_INITIALIZING    = 2,
    DUNESTBAPI_VMDVB_STATE_INITIALIZED     = 3,
    DUNESTBAPI_VMDVB_STATE_DEINITIALIZING  = 4,

    DUNESTBAPI_VMDVB_STATE_NUMBER
}
dunestbapi_vmDvbState;

typedef enum
{
    DUNESTBAPI_VMDVB_EVENT_NO_EVENT    = 1,

    // Other Verimatrix DVB client errors.
    DUNESTBAPI_VMDVB_EVENT_ERROR_OTHER = 2,

    DUNESTBAPI_VMDVB_EVENT_NUMBER
}
dunestbapi_vmDvbEvent;

typedef void (*dunestbapi_vmDvbEventCallback)(
    dunestbapi_vmDvbState in_previousState,
    dunestbapi_vmDvbState in_currentState,
    dunestbapi_vmDvbEvent in_event,
    int in_vmdvb_error_code);

// The callback may be called in other thread.
// Pass NULL to disable callback.
dunestbapi_result dunestbapi_setVmDvbEventCallback(
    dunestbapi_vmDvbEventCallback in_callback);

typedef struct
{
    int smartCardUsed;
    char *smartCardNumber;
    int caSystemId;
}
dunestbapi_vmDvbCardStatus;

typedef void (*dunestbapi_vmDvbCardStatusCallback)(
    const dunestbapi_vmDvbCardStatus *in_currentCardStatus);

// The callback may be called in other thread.
// Pass NULL to disable callback.
dunestbapi_result dunestbapi_setVmDvbCardStatusCallback(
    dunestbapi_vmDvbCardStatusCallback in_callback);

typedef void (*dunestbapi_vmDvbDescramblingCallback)(
    int in_descramblingOn);

// The callback may be called in other thread.
// Pass NULL to disable callback.
dunestbapi_result dunestbapi_setVmDvbDescramblingCallback(
    dunestbapi_vmDvbDescramblingCallback in_callback);

typedef void (*dunestbapi_vmDvbEcmEmmCountersCallback)(
    int in_ecmCounter, int in_emmCounter);

// The callback may be called in other thread.
// Pass NULL to disable callback.
dunestbapi_result dunestbapi_setVmDvbEcmEmmCountersCallback(
    dunestbapi_vmDvbEcmEmmCountersCallback in_callback);

dunestbapi_result dunestbapi_getVmDvbCardStatus(
    dunestbapi_vmDvbCardStatus **out_vmDvbCardStatus);

dunestbapi_result dunestbapi_isVmDvbDescramblingOn(
    int *out_descramblingOn);

dunestbapi_result dunestbapi_getVmDvbEcmEmmCounters(
    int *out_ecmCounter, int *out_emmCounter);

dunestbapi_result dunestbapi_getVmDvbBoxId(
    char **out_boxId);

dunestbapi_result dunestbapi_getVmDvbLibraryVersion(
    char **out_libraryVersion);

typedef enum
{
    DUNESTBAPI_VMDVB_PAIRING_NONE        = 1,
    DUNESTBAPI_VMDVB_PAIRING_IN_PROGRESS = 2,
    DUNESTBAPI_VMDVB_PAIRING_OK          = 3,
    DUNESTBAPI_VMDVB_PAIRING_BAD         = 4,

    DUNESTBAPI_VMDVB_PAIRING_NUMBER
}
dunestbapi_vmDvbPairing;

dunestbapi_result dunestbapi_getVmDvbPairing(
    dunestbapi_vmDvbPairing *out_pairing);

#define DUNESTBAPI_VMDVB_CURRENCY_SIGN_LOCATION_BEGINNING 1
#define DUNESTBAPI_VMDVB_CURRENCY_SIGN_LOCATION_END       2

typedef struct
{
    int tokenMult;
    int tokenDiv;
    int currencySignLocation;
    char *centsDelimiter;
    char *thousandsDelimiter;
    int centsDigits;
    char *currencyText;
}
dunestbapi_vmDvbPursePresentationProperties;

dunestbapi_result dunestbapi_getVmDvbCurrentPursePresentationProperties(
    dunestbapi_vmDvbPursePresentationProperties **out_pursePresentationProperties);

dunestbapi_result dunestbapi_getVmDvbCurrentPurseValues(
    long long **out_values, int *out_valuesNumber);

#define DUNESTBAPI_VMDVB_PIN_TYPE_PARENTAL_CONTROL               1
#define DUNESTBAPI_VMDVB_PIN_TYPE_IPPV                           2
#define DUNESTBAPI_VMDVB_PIN_TYPE_PARENTAL_CONTROL_USING_NSC_PIN 3
#define DUNESTBAPI_VMDVB_PIN_TYPE_RESUMING_EVENT                 4
#define DUNESTBAPI_VMDVB_PIN_TYPE_SELECTING_EVENT                5
#define DUNESTBAPI_VMDVB_PIN_TYPE_INTERNAL_FOR_SMART_CARD        6
#define DUNESTBAPI_VMDVB_PIN_TYPE_INTERNAL_FOR_NON_SMART_CARD    7

typedef struct
{
    int pinType;
    int pinIndex;
    int serviceType;
}
dunestbapi_vmDvbPinParameters;

typedef void (*dunestbapi_vmDvbPinRequestCallback)(
    const dunestbapi_vmDvbPinParameters *in_pinRequestParameters);

// The callback may be called in other thread.
// Pass NULL to disable callback.
dunestbapi_result dunestbapi_setVmDvbPinRequestCallback(
    dunestbapi_vmDvbPinRequestCallback in_callback);

typedef struct
{
    long long eventId; // Not used anywhere now.
}
dunestbapi_vmDvbOrderPinRequestExtraParameters;

// ... more comments
// available_credit = in_purse * in_tokenMult / in_tokenDiv
// cost_of_event = in_cost * in_tokenMult / in_tokenDiv
// ... more comments
// Use DUNESTBAPI_VMDVB_PIN_TYPE_IPPV as PIN type in
// dunestbapi_vmDvbCheckPinInput().
// ... more comments
// pinType in the first argument is DUNESTBAPI_VMDVB_PIN_TYPE_IPPV.
typedef void (*dunestbapi_vmDvbOrderPinRequestCallback)(
    const dunestbapi_vmDvbPinParameters *in_orderPinRequestParameters,
    long long in_purse,
    long long in_cost,
    const dunestbapi_vmDvbPursePresentationProperties *in_pursePresentationProperties,
    const dunestbapi_vmDvbOrderPinRequestExtraParameters *in_extraParameters);

dunestbapi_result dunestbapi_setVmDvbOrderPinRequestCallback(
    dunestbapi_vmDvbOrderPinRequestCallback in_callback);

dunestbapi_result dunestbapi_vmDvbInputPin(
    const char *in_pin,
    const dunestbapi_vmDvbPinParameters *in_pinInputParameters);

// Used for both checking and changing PIN.
typedef enum
{
    DUNESTBAPI_VMDVB_PIN_INPUT_VERIFIED                      = 1,
    DUNESTBAPI_VMDVB_PIN_INPUT_CHECK_FAILED_NOT_ENOUGH_PURSE = 2,
    DUNESTBAPI_VMDVB_PIN_INPUT_CHECK_FAILED_BLOCKED          = 3,
    DUNESTBAPI_VMDVB_PIN_INPUT_CHECK_FAILED_WRONG_PIN        = 4,

    DUNESTBAPI_VMDVB_PIN_INPUT_NUMBER
}
dunestbapi_vmDvbPinInputOperationResult;

typedef void (*dunestbapi_vmDvbPinInputOperationResultCallback)(
    dunestbapi_vmDvbPinInputOperationResult in_result);

// The callback may be called in other thread.
// Pass NULL to disable callback.
// ... comments?
dunestbapi_result dunestbapi_setVmDvbPinInputOperationResultCallback(
    dunestbapi_vmDvbPinInputOperationResult in_callback);

#define DUNESTBAPI_VMDVB_PIN_CHANGE_TYPE_STB_LOCK      1
#define DUNESTBAPI_VMDVB_PIN_CHANGE_TYPE_HOME_SHOPPING 2

typedef struct
{
    char *oldPin;
    char *newPin;
    int pinChangeType;
}
dunestbapi_vmDvbPinChangeParameters;

dunestbapi_result dunestbapi_vmDvbChangePin(
    const dunestbapi_vmDvbPinChangeParameters *in_pinChangeParameters);

#define DUNESTBAPI_VMDVB_OSM_DISPLAY_MODE_FX_NEXTSTB 0x01
#define DUNESTBAPI_VMDVB_OSM_DISPLAY_MODE_RM_NEXTSTB 0x02
#define DUNESTBAPI_VMDVB_OSM_DISPLAY_MODE_FX_NEXTCA  0x03
#define DUNESTBAPI_VMDVB_OSM_DISPLAY_MODE_RM_NEXTCA  0x04
#define DUNESTBAPI_VMDVB_OSM_DISPLAY_MODE_FX_STILLCA 0x05
#define DUNESTBAPI_VMDVB_OSM_DISPLAY_MODE_RM_STILLCA 0x06
#define DUNESTBAPI_VMDVB_OSM_DISPLAY_MODE_FX_IMMED   0x07
#define DUNESTBAPI_VMDVB_OSM_DISPLAY_MODE_RM_IMMED   0x08

#define DUNESTBAPI_VMDVB_OSM_ALIGNMENT_LEFT   0x0
#define DUNESTBAPI_VMDVB_OSM_ALIGNMENT_RIGHT  0x1
#define DUNESTBAPI_VMDVB_OSM_ALIGNMENT_CENTER 0x2
#define DUNESTBAPI_VMDVB_OSM_ALIGNMENT_ALL    0x3

// TODO: comments.
typedef struct
{
    int displayMode;
    int duration;
    int xOriginLeft;
    int xAnchorCenter;
    int xPosition;
    int yOriginTop;
    int yAnchorCenter;
    int yPosition;
    int textAlign;
    int bgColor;
    int alpha;
}
dunestbapi_vmDvbOsmPresentationProperties;

// TODO: comments.
typedef void (*dunestbapi_vmDvbOsmCallback)(
    int in_messageId,
    const char *in_text,
    const dunestbapi_vmDvbOsmPresentationProperties *in_presentationProperties);

dunestbapi_result dunestbapi_setVmDvbOsmCallback(
    dunestbapi_vmDvbOsmCallback in_callback);

// TODO: comments.
typedef void (*dunestbapi_vmDvbTriggerCallback)(
    const char *in_chars,
    const unsigned char *in_bytes,
    int in_bytesSize);

dunestbapi_result dunestbapi_setVmDvbTriggerCallback(
    dunestbapi_vmDvbTriggerCallback in_callback);

typedef struct
{
    int period;
}
dunestbapi_vmDvbWatermarkingParameters;

typedef void (*dunestbapi_vmDvbWatermarkingCallback)(
    const dunestbapi_vmDvbWatermarkingParameters *in_watermarkingParameters);

dunestbapi_result dunestbapi_setVmDvbWatermarkingCallback(
    dunestbapi_vmDvbWatermarkingCallback in_callback);

// There is no STB-visible customer binding API.

typedef struct
{
    int originalNetworkId;
    int serviceId;
    char *easCode;
}
dunestbapi_vmDvbEasBeginParameters;

// It is not clear how it should work but probably there can be several EAS
// in parallel.
typedef void (*dunestbapi_vmDvbEasBeginCallback)(
    long long in_id,
    const dunestbapi_vmDvbEasBeginParameters *in_EasBeginParameters);

dunestbapi_result dunestbapi_setVMDvbEasBeginCallback(
    dunestbapi_vmDvbEasBeginCallback in_callback);

typedef void (*dunestbapi_vmDvbEasEndCallback)(
    long long in_id);

dunestbapi_result dunestbapi_setVMDvbEasEndCallback(
    dunestbapi_vmDvbEasEndCallback in_callback);

///////////////////////////////////////////////////////////////////////////

//
// DVB.
//

dunestbapi_result dunestbapi_enableEpgFromMpegTsStream(void);

dunestbapi_result dunestbapi_disableEpgFromMpegTsStream(void);

dunestbapi_result dunestbapi_getEpgFromMpegTsStream(
    const char *in_channelUrl,
    const dunestbapi_timeRange *in_timeRange,
    dunestbapi_epgItem **out_epgItemsDescription,
    int *out_epgItemsNumber);

dunestbapi_result dunestbapi_getChannelsNumber(int *out_number);

dunestbapi_result dunestbapi_getChannelsDescription(
    const int in_startIndex, const int in_maxCount,
    dunestbapi_channel **out_channelsDescription,
    int *out_channelsNumber);

dunestbapi_result dunestbapi_getChannelGroups(char ***out_channelGroupUrls,
    int *out_channelGroupUrlsNumber);

dunestbapi_result dunestbapi_getChannelsNumberByChannelGroup(
    const char *in_channelGroupUrl, int *out_number);

dunestbapi_result dunestbapi_getChannelsDescriptionByChannelGroup(
    const char *in_channelGroupUrl, const int in_startIndex,
    const int in_maxCount, dunestbapi_channel **out_channelsDescription,
    int *out_channelsNumber);

dunestbapi_result dunestbapi_getChannelGroupDescription(
    const char *in_channelGroupUrl,
    dunestbapi_channelGroup **out_channelGroupDescription);

//
// DVB-S scanner.
//

dunestbapi_result dunestbapi_startDvbsScan(void);

dunestbapi_result dunestbapi_cancelDvbsScan(void);

#define DUNESTBAPI_TRANSPONDER_POLARIZATION_HORZ_OR_LEFT  0
#define DUNESTBAPI_TRANSPONDER_POLARIZATION_VERT_OR_RIGHT 1
#define DUNESTBAPI_TRANSPONDER_POLARIZATION_LEFT          2
#define DUNESTBAPI_TRANSPONDER_POLARIZATION_RIGHT         3
#define DUNESTBAPI_TRANSPONDER_POLARIZATION_HORZ          4
#define DUNESTBAPI_TRANSPONDER_POLARIZATION_VERT          5

dunestbapi_result dunestbapi_getDvbsScannerStatus(
    dunestbapi_dvbsScannerStatus **out_dvbsScannerStatus);

dunestbapi_result dunestbapi_setDvbsScannerCallback(
    dunestbapi_dvbsScannerCallback in_callback);

//
// DVB-C scanner.
//

dunestbapi_result dunestbapi_startDvbcScan(void);

dunestbapi_result dunestbapi_cancelDvbcScan(void);

dunestbapi_result dunestbapi_getDvbcScannerStatus(
    dunestbapi_dvbcScannerStatus **out_dvbcScannerStatus);

dunestbapi_result dunestbapi_setDvbcScannerCallback(
    dunestbapi_dvbcScannerCallback in_callback);

///////////////////////////////////////////////////////////////////////////

//
// Picture in picture (PiP).
//

dunestbapi_result dunestbapi_playPip(const char *in_mediaUrl);

dunestbapi_result dunestbapi_stopPip(void);

dunestbapi_result dunestbapi_isPipWindowFullScreen(
    int *out_windowFullScreen);

dunestbapi_result dunestbapi_enablePipWindowFullScreen(void);

dunestbapi_result dunestbapi_getPipWindowRect(int *out_x, int *out_y,
    int *out_width, int *out_height);

dunestbapi_result dunestbapi_setPipWindowRect(const int in_x,
    const int in_y, const int in_width, const int in_height);

dunestbapi_result dunestbapi_isPipEnabled(int *out_pipEnabled);

dunestbapi_result dunestbapi_enablePip(void);

dunestbapi_result dunestbapi_disablePip(void);

dunestbapi_result dunestbapi_getPipAlphaLevel(int *out_level);

dunestbapi_result dunestbapi_setPipAlphaLevel(const int in_level);

dunestbapi_result dunestbapi_getPipZOrder(int *out_zOrder);

dunestbapi_result dunestbapi_setPipZOrder(const int in_zOrder);

dunestbapi_result dunestbapi_getPipPlaybackState(
    dunestbapi_playbackState *out_playbackState);

// The callback is called on internal STB thread.
// Pass NULL to disable PiP playback event callback.
dunestbapi_result dunestbapi_setPipPlaybackEventCallback(
    dunestbapi_playbackEventCallback in_callback);

///////////////////////////////////////////////////////////////////////////

//
// Utility functions for various types.
//

void dunestbapi_printStringsArray(
    const char *in_prefix,
    const char *in_name,
    char **in_strings,
    const int in_number);

void dunestbapi_freeStringsArray(
    char **in_strings,
    const int in_number);

void dunestbapi_printNetworkConfigurationStruct(
    const char *in_prefix,
    const dunestbapi_networkConfiguration *in_networkConfiguration);

void dunestbapi_freeNetworkConfigurationStruct(
    dunestbapi_networkConfiguration *in_networkConfiguration);

void dunestbapi_printInterfaceStatusArray(
    const char *in_prefix,
    const dunestbapi_interfaceStatus *in_interfaces,
    const int in_interfacesNumber);

void dunestbapi_freeInterfaceStatusArray(
    dunestbapi_interfaceStatus *in_interfaces,
    const int in_interfacesNumber);

void dunestbapi_printWifiNetworkArray(
    const char *in_prefix,
    const dunestbapi_wifiNetwork *in_wifiNetworks,
    const int in_wifiNetworksNumber);

void dunestbapi_freeWifiNetworkArray(
    dunestbapi_wifiNetwork *in_wifiNetworks,
    const int in_wifiNetworksNumber);

void dunestbapi_printStorageStruct(
    const char *in_prefix,
    const dunestbapi_storage *in_storage);

void dunestbapi_freeStorageStruct(
    dunestbapi_storage *in_storage);

void dunestbapi_printFolderContentStruct(
    const char *in_prefix,
    const dunestbapi_folderContent *in_folderContent);

void dunestbapi_freeFolderContentStruct(
    dunestbapi_folderContent *in_folderContent);

void dunestbapi_printEpgItemArray(
    const char *in_prefix,
    const dunestbapi_epgItem *in_epgItems,
    const int in_epgItemsNumber);

void dunestbapi_freeEpgItemArray(
    dunestbapi_epgItem *in_epgItems,
    const int in_epgItemsNumber);

void dunestbapi_printChannelArray(
    const char *in_prefix,
    const dunestbapi_channel *in_channels,
    const int in_channelsNumber);

void dunestbapi_freeChannelArray(
    dunestbapi_channel *in_channels,
    const int in_channelsNumber);

void dunestbapi_printChannelGroupStruct(
    const char *in_prefix,
    const dunestbapi_channelGroup *in_channelGroup);

void dunestbapi_freeChannelGroupStruct(
    dunestbapi_channelGroup *in_channelGroup);

void dunestbapi_printDvbsScannerStatusStruct(
    const char *in_prefix,
    const dunestbapi_dvbsScannerStatus *in_dvbsScannerStatus);

void dunestbapi_freeDvbsScannerStatusStruct(
    dunestbapi_dvbsScannerStatus *in_dvbsScannerStatus);

void dunestbapi_printDvbcScannerStatusStruct(
    const char *in_prefix,
    const dunestbapi_dvbcScannerStatus *in_dvbcScannerStatus);

void dunestbapi_freeDvbcScannerStatusStruct(
    dunestbapi_dvbcScannerStatus *in_dvbcScannerStatus);

void dunestbapi_printRecordingParametersStruct(
    const char *in_prefix,
    const dunestbapi_recordingParameters *in_recordingParameters);

void dunestbapi_freeRecordingParametersStruct(
    dunestbapi_recordingParameters *in_recordingParameters);

void dunestbapi_printRecordingStatusStruct(
    const char *in_prefix,
    const dunestbapi_recordingStatus *in_recording);

void dunestbapi_freeRecordingStatusStruct(
    dunestbapi_recordingStatus *in_recording);

void dunestbapi_printRecordingStatusArray(
    const char *in_prefix,
    const dunestbapi_recordingStatus *in_recordings,
    const int in_recordingsNumber);

void dunestbapi_freeRecordingStatusArray(
    dunestbapi_recordingStatus *in_recordings,
    const int in_recordingsNumber);

void dunestbapi_printRecordingFilterStruct(
    const char *in_prefix,
    const dunestbapi_recordingFilter *in_recordingFilter);

void dunestbapi_freeRecordingFilterStruct(
    dunestbapi_recordingFilter *in_recordingFilter);

void dunestbapi_printVmDvbCardStatusStruct(
    const char *in_prefix,
    const dunestbapi_vmDvbCardStatus *in_vmDvbCardStatus);

void dunestbapi_freeVmDvbCardStatusStruct(
    dunestbapi_vmDvbCardStatus *in_vmDvbCardStatus);

void dunestbapi_printVmDvbOsmPresentationProperties(
    const char *in_prefix,
    const dunestbapi_vmDvbOsmPresentationProperties *in_vmDvbOsmPresentationProperties);

void dunestbapi_freeVmDvbOsmPresentationProperties(
    dunestbapi_vmDvbOsmPresentationProperties *in_vmDvbOsmPresentationProperties);

///////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
}
#endif

#endif // DUNE_STB_API_H
